@@ -4,7 +4,7 @@
 #' A key showing how the colors map to raster values is shown below the map.
 #' The width and height of the graphics region will be automagically determined in some cases.
 #'
-#' @param r 'Raster*', 'SpatialGridDataFrame', 'SpatialPixelsDataFrame', or 'CRS'.
+#' @param r 'Raster*', 'Spatial*', or 'CRS'.
 #'   An object that can be converted to a raster layer, or a coordinate reference system (CRS).
 #' @param p 'SpatialPointsDataFrame'.
 #'   Spatial point data to be plotted.
@@ -12,8 +12,8 @@
 #'   Graphics parameters to be passed to \code{\link{AddPoints}}.
 #'   Unused if \code{p = NULL}.
 #' @param layer 'integer'.
-#'   Layer to extract from if \code{r} is of class RasterStack/Brick or SpatialGridDataFrame.
-#' @param att integer or character.
+#'   Layer to extract from if \code{r} is of class 'RasterStack/Brick' or 'SpatialGridDataFrame'.
+#' @param att 'integer' or 'character'.
 #'   The levels attribute to use in the Raster Attribute Table (RAT);
 #'   requires \code{r} values of class factor.
 #' @param n 'integer'.
@@ -62,10 +62,11 @@
 #'   This list may include components \code{at} and \code{labels}.
 #' @param scale.loc 'character'.
 #'   Position of the scale bar:
-#'   "bottomleft", "topleft", "topright", or "bottomright" to denote scale location.
+#'   \code{"bottomleft"}, \code{"topleft"}, \code{"topright"}, or \code{"bottomright"} to denote scale location,
+#'   see \code{\link{AddScaleBar}} function.
 #' @param arrow.loc 'character'.
 #'   Position of the north arrow:
-#'   "bottomleft", "topleft", "topright", or "bottomright" to denote arrow location.
+#'   \code{"bottomleft"}, \code{"topleft"}, \code{"topright"}, or \code{"bottomright"} to denote arrow location.
 #' @param explanation 'character'.
 #'   Label explaining the raster cell value.
 #' @param credit 'character'.
@@ -95,7 +96,7 @@
 #'   Passed arguments include \code{"x"}, \code{"col"}, and \code{"lwd"}.
 #' @param draw.key 'logical'.
 #'   If true, a color key should be drawn.
-#' @param draw.raster logical.
+#' @param draw.raster 'logical'.
 #'   If true, the raster image is drawn.
 #' @param file 'character'.
 #'   Name of the output file.
@@ -108,20 +109,33 @@
 #'   If true, the graphics device driver is shut down when the function exits.
 #'   Unused if \code{file = NULL}
 #' @param useRaster 'logical'.
-#'   If true, a bitmap raster is used to plot \code{r} instead of using polygons.
+#'   If true, a bitmap raster is used to plot \code{r} instead of using individual polygons for each raster cell.
 #'   If \code{UseRaster} is not specified, raster images are used when the \code{getOption("preferRaster")} is true.
+#'   Unused if \code{simplify = TRUE}.
+#' @param simplify 'numeric'.
+#'   Specifying this argument will convert the raster \code{r} to spatial polygons prior to plotting,
+#'   see \code{\link{Grid2Polygons}} function for details.
+#'   If \code{simplify > 0} the geometry of the spatial polygons is generalized using the
+#'   Douglas-Peucker algorithm (Douglas and Peucker, 1961);
+#'   and \code{simplify} is the numerical tolerance value to be used by the algorithm.
+#'   See \code{\link[rgeos]{gSimplify}} function for additional information.
 #'
 #' @return Used for the side-effect of a new plot generated.
 #'   Returns a 'list' object with the following graphical parameters:
 #'   \describe{
-#'     \item{din}{device dimensions \code{(width, height)}, in inches.}
-#'     \item{usr}{extremes of the coordinates of the plotting region \code{(x1, x2, y1, y2)}.}
-#'     \item{heights}{relative heights on the device \code{(upper, lower)} for the map and color-key plots.}
+#'     \item{din}{device dimensions \code{c(width, height)}, in inches.}
+#'     \item{usr}{extremes of the coordinates of the plotting region \code{c(x1, x2, y1, y2)}.}
+#'     \item{heights}{relative heights on the device \code{c(upper, lower)} for the map and color-key plots.}
 #'   }
 #'
 #' @author J.C. Fisher, U.S. Geological Survey, Idaho Water Science Center
 #'
-#' @seealso \code{\link{AddScaleBar}}, \code{\link{AddColorKey}}
+#' @references
+#'   Douglas, D., and Peucker, T., 1961,
+#'   Algorithms for the reduction of the number of points required to represent a digitized line or its caricature:
+#'   The Canadian Cartographer, v. 10, no. 2, p. 112--122.
+#'
+#' @seealso \code{\link{AddColorKey}}
 #'
 #' @keywords hplot
 #'
@@ -142,25 +156,36 @@
 #' rat$code <- c(12, 25, 30)
 #' levels(r) <- rat
 #' PlotMap(r, att = "land.cover", col = c("grey", "orange", "purple"))
+#'
 #' PlotMap(r, att = "code")
 #'
-#' r <- raster::raster(system.file("external/test.grd", package="raster"))
-#' credit <- "Label crediting the map."
-#' explanation <- "Label explaining the raster cell value."
-#' PlotMap(r, scale.loc = "bottomright", dms.tick = TRUE, credit = credit,
-#'         explanation = explanation)
+#' m <- t(datasets::volcano)[61:1, ]
+#' x <- seq(from = 6478705, length.out = 87, by = 10)
+#' y <- seq(from = 2667405, length.out = 61, by = 10)
+#' r <- raster::raster(m, xmn = min(x), xmx = max(x), ymn = min(y), ymx = max(y),
+#'                     crs = "+init=epsg:27200")
+#' PlotMap(r, pal = terrain.colors, scale.loc = "bottomright",
+#'         explanation = "Topographic information on Auckland's Maunga Whau volcano.",
+#'         credit = "Digitized from a topographic map by Ross Ihaka on a 10-m by 10-m grid.",
+#'         shade = list(alpha = 0.3), contour.lines = list(col = "#1F1F1F"),
+#'         useRaster = TRUE)
+#'
+#' r <- raster::raster(system.file("external/test.grd", package = "raster"))
+#' Pal <- colorspace::rainbow_hcl
+#' breaks <- seq(0, 2000, by = 200)
+#' PlotMap(r, breaks = breaks, pal = Pal, dms.tick = TRUE, bg.lines = TRUE,
+#'         scale.loc = "bottomright", contour.lines = list(col = "#1F1F1F"),
+#'         draw.key = FALSE, simplify = 0)
+#' AddGradientLegend(breaks, Pal, at = breaks, title = "Explanation", loc = "topleft",
+#'                   inset = c(0.1, 0.1), strip.dim = c(2, 20))
+#'
+#' out <- PlotMap(r, dms.tick = TRUE, file = "Rplots1.pdf")
+#'
+#' pdf(file = "Rplots2.pdf", width = out$din[1], height = out$din[2])
+#' PlotMap(r, dms.tick = TRUE)
 #' data(meuse, package = "sp")
 #' sp::coordinates(meuse) = ~ x + y
 #' points(meuse)
-#'
-#' val <- PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, credit = credit,
-#'                explanation = explanation, file = "Rplots1.pdf")
-#' print(val)
-#'
-#' pdf(file = "Rplots2.pdf", width = val$din[1], height = val$din[2])
-#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, credit = credit,
-#'         explanation = explanation)
-#' points(meuse)
 #' dev.off()
 #'
 #' file.remove(c("Rplots1.pdf", "Rplots2.pdf"))
@@ -174,32 +199,33 @@ PlotMap <- function(r, p=NULL, ..., layer=1, att=NULL, n=NULL, breaks=NULL,
                     bg.image.alpha=1, pal=NULL, col=NULL, max.dev.dim=c(43, 56),
                     labels=NULL, scale.loc=NULL, arrow.loc=NULL, explanation=NULL,
                     credit=NULL, shade=NULL, contour.lines=NULL,
-                    rivers=NULL, lakes=NULL, roads=NULL, draw.key=NULL,
-                    draw.raster=TRUE, file=NULL, close.file=TRUE, useRaster) {
+                    rivers=NULL, lakes=NULL, roads=NULL, draw.key=NULL, draw.raster=TRUE,
+                    file=NULL, close.file=TRUE, useRaster, simplify) {
 
   if (!is.null(p) && !inherits(p, "SpatialPoints"))
-    stop("spatial point data is the incorrect class")
+    stop("spatial point data is invalid class")
 
   if (!is.null(bg.image) && !inherits(bg.image, "RasterLayer"))
-    stop("background image is the incorrect class")
+    stop("background image is invalid class")
 
   if (missing(useRaster)) {
     useRaster <- getOption("preferRaster")
     if (!is.logical(useRaster)) useRaster <- FALSE
   }
 
-  if (inherits(r, "CRS")) {
-    is.lim <- is.numeric(xlim) && length(xlim) == 2 && all(!is.na(xlim)) &&
-              is.numeric(ylim) && length(ylim) == 2 && all(!is.na(ylim))
+  if (inherits(r, c("RasterStack", "RasterBrick", "SpatialGrid", "SpatialPixelsDataFrame"))) {
+    r <- raster(r, layer=layer)
+  } else if (inherits(r, "CRS")) {
+    is.lim <- !is.null(xlim) && !is.null(ylim)
     if (!is.lim && is.null(bg.image)) stop("spatial limits must be specified")
     e <- extent(if (is.lim) c(xlim, ylim) else bg.image)
     r <- raster(e, crs=r)
     r[] <- NA
   }
-
-  if (inherits(r, c("RasterStack", "RasterBrick", "SpatialGrid", "SpatialPixelsDataFrame")))
-    r <- raster(r, layer=layer)
-  if (!inherits(r, "RasterLayer")) stop("raster layer is the incorrect class")
+  if (!inherits(r, "RasterLayer")) {
+    r <- raster(r)
+    r[] <- NA
+  }
 
   if (!is.null(p)) try(p <- spTransform(p, r@crs), silent=TRUE)
 
@@ -458,8 +484,19 @@ PlotMap <- function(r, p=NULL, ..., layer=1, att=NULL, n=NULL, breaks=NULL,
   }
 
   if (draw.raster & n > 0) {
-    raster::image(r, maxpixels=length(r), useRaster=useRaster, zlim=zl,
-                  col=cols, add=TRUE, breaks=breaks)
+    if (!missing(simplify)) {
+      ply <- Grid2Polygons(r, level=TRUE, at=breaks, zlim=zl)
+      if (simplify > 0) {
+        simple.ply <- rgeos::gSimplify(ply, tol=simplify, topologyPreserve=TRUE)
+        FUN <- function(i) methods::slot(i, "ID")
+        ids <- sapply(methods::slot(simple.ply, "polygons"), FUN)
+        ply <- sp::SpatialPolygonsDataFrame(simple.ply, data=ply[ids, ]@data)
+      }
+      plot(ply, col=cols, border=NA, add=TRUE)
+    } else {
+      raster::image(r, maxpixels=length(r), useRaster=useRaster, zlim=zl,
+                    col=cols, add=TRUE, breaks=breaks)
+    }
     if (is.list(shade)) {
       zfact <- as.numeric(shade[["z.factor"]])
       angle <- as.numeric(shade[["angle"]])
@@ -523,7 +560,6 @@ PlotMap <- function(r, p=NULL, ..., layer=1, att=NULL, n=NULL, breaks=NULL,
     drawl <- ifelse(length(drawl) == 1 && !is.na(drawl), drawl, TRUE)
     metho <- ifelse(length(metho) == 1 && !is.na(metho), metho, "flattest")
     contour.breaks <- if (n + 1L > 20L) pretty(zl, 20L) else breaks
-    ncontours <- length(contour.breaks)
     raster::contour(r, maxpixels=length(r), levels=contour.breaks,
                     labels=formatC(contour.breaks, big.mark=","),
                     xlim=xl, ylim=yl, zlim=zl, labcex=0.5, drawlabels=drawl,
